package org.ta.parsers;


public class TAFileParseException
extends Exception {
    private static final long serialVersionUID = 1;

    public TAFileParseException(String string) {
        super(string);
    }
}

